Package: rhdf5client
Title: Access HDF5 content from HDF Scalable Data Service
Description: This package provides functionality for reading data from HDF Scalable Data Service
	from within R.  The HSDSArray function bridges from HSDS to the user via the DelayedArray
	interface.  Bioconductor manages an open HSDS instance graciously provided by John
	Readey of the HDF Group.
Version: 1.34.2
Authors@R: c(
 person("Samuela", "Pollack", role = c("aut"),email = "spollack@jimmy.harvard.edu"), 
 person("Shweta", "Gopaulakrishnan", role = c("aut"), email = "reshg@channing.harvard.edu"),
 person("BJ", "Stubbs", role = c("aut"), email = "rebjh@channing.harvard.edu"),
 person("Alexey", "Sergushichev", role = c("aut", "cre"), email = "alsergbox@gmail.com"),
 person("Vincent", "Carey", role = c("aut"), email = "stvjc@channing.harvard.edu")
 )
Suggests: knitr, testthat, BiocStyle, DT, rmarkdown
Imports: httr, rjson, utils, data.table
Depends: R (>= 3.6), methods, DelayedArray
License: Artistic-2.0
LazyLoad: yes
Collate: URL_hsds.R indx.R 'rhdf5client2.R' 'Source.R' 'File.R'
        'Dataset.R' 'RHDF5Array.R'
VignetteBuilder: knitr
Encoding: UTF-8
biocViews: DataImport, Software, Infrastructure
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/rhdf5client
git_branch: RELEASE_3_23
git_last_commit: 0ed5c74
git_last_commit_date: 2026-06-01
Repository: Bioconductor 3.23
Date/Publication: 2026-06-02
NeedsCompilation: yes
Packaged: 2026-06-02 23:55:44 UTC; biocbuild
Author: Samuela Pollack [aut],
  Shweta Gopaulakrishnan [aut],
  BJ Stubbs [aut],
  Alexey Sergushichev [aut, cre],
  Vincent Carey [aut]
Maintainer: Alexey Sergushichev <alsergbox@gmail.com>
