# Generated by roxygen2: do not edit by hand

export(HSDSArray)
export(HSDSDataset)
export(HSDSFile)
export(HSDSSource)
export(URL_hsds)
export(check_hsds)
export(extractCompoundJSON)
export(getData)
export(getHSDSFileList)
export(isplit)
export(listDatasets)
export(listDomains)
export(sproc)
exportClasses(HSDSArray)
exportClasses(HSDSMatrix)
import(DelayedArray)
import(httr)
import(methods)
import(rjson)
importFrom(DelayedArray,new_DelayedArray)
importFrom(data.table,as.data.table)
importFrom(data.table,rbindlist)
importFrom(utils,strcapture)
useDynLib(rhdf5client, .registration = TRUE)
