#' rhdf5client: A package for accessing HDFGroup HDF5 servers from R.
#'
#' The rhdf5client package provides read-only access to HDF5 files
#' maintained on a server. The HDFGroup provides two servers, an
#' obsolescent one called `h5serv` and the newer prototype called `hsds`.  
#' @docType package
#' @name rhdf5client
NULL

#' @import methods
