% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/File.R
\docType{class}
\name{HSDSFile-class}
\alias{HSDSFile-class}
\title{An S4 class to represent an HDF5 file accessible from a server.}
\description{
An S4 class to represent an HDF5 file accessible from a server.
}
\section{Slots}{

\describe{
\item{\code{HSDSSource}}{an object of type HSDSSource}

\item{\code{domain}}{the file's domain on the server; more or less, an alias for its 
location in the external server file system}

\item{\code{dsetdf}}{a data.frame that caches often-used information about the file}

\item{\code{type}}{string representing type of the file: "domain" for file and "folder" for directory}
}}

