% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RHDF5Array.R
\name{check_hsds}
\alias{check_hsds}
\title{a test request}
\usage{
check_hsds()
}
\value{
logical, TRUE if hsds behaving as expected
}
\description{
a test request
}
\examples{
check_hsds()
}
