% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/URL_hsds.R
\name{URL_hsds}
\alias{URL_hsds}
\title{manage hsds URL}
\usage{
URL_hsds()
}
\value{
URL of hsds server
}
\description{
manage hsds URL
}
\examples{
URL_hsds()
}
